@@ -91,6 +91,7 @@ class BaseFilter:
         wtsheet_name - the name of the sheet into which content
                        should be written.
         """
+        self.rdsheet = rdsheet
         self.next.sheet(rdsheet,wtsheet_name)
        
     def set_rdsheet(self,rdsheet):
@@ -103,6 +104,7 @@ class BaseFilter:
                   this point forward should be read from.
 
         """
+        self.rdsheet = rdsheet
         self.next.set_rdsheet(rdsheet)
        
     def row(self,rdrowx,wtrowx):
@@ -198,10 +200,10 @@ class BaseWriter:
     def workbook(self,rdbook,wtbook_name):
         self.close()        
         self.rdbook = rdbook
-        self.wtbook = xlwt.Workbook()
+        self.wtbook = xlwt.Workbook(style_compression=2)
         self.wtname = wtbook_name
         self.style_list = []
-        self.wtsheets = {}
+        self.wtsheet_names = set()
         for rdxf in rdbook.xf_list:
             wtxf = xlwt.Style.XFStyle()
             #
@@ -277,6 +279,13 @@ class BaseWriter:
             self.style_list.append(wtxf)
    
     def sheet(self,rdsheet,wtsheet_name):
+        # these checks should really be done by xlwt!
+        if not wtsheet_name:
+            raise ValueError('Empty sheet name will result in invalid Excel file!')
+        l_wtsheet_name = wtsheet_name.lower()
+        if l_wtsheet_name in self.wtsheet_names:
+            raise ValueError('A sheet named %r has already been added!'%l_wtsheet_name)
+        self.wtsheet_names.add(l_wtsheet_name)
         self.rdsheet = rdsheet
         self.wtsheet_name=wtsheet_name
         self.wtsheet = wtsheet = self.wtbook.add_sheet(wtsheet_name)
@@ -433,6 +442,15 @@ class GlobReader(BaseReader):
     def get_filepaths(self):
         return glob(self.spec)
 
+class XLRDReader(BaseReader):
+
+    def __init__(self,wb,filename):
+        self.wb = wb
+        self.filename = filename
+        
+    def get_workbooks(self):
+        yield (self.wb,self.filename)
+
 class DirectoryWriter(BaseWriter):
 
     def __init__(self,path):
@@ -441,6 +459,20 @@ class DirectoryWriter(BaseWriter):
     def get_stream(self,filename):
         return file(os.path.join(self.dir_path,filename),'wb')
 
+class StreamWriter(BaseWriter):
+
+    fired = False
+    close_after_write = False
+    
+    def __init__(self,stream):
+        self.stream = stream
+        
+    def get_stream(self,filename):
+        if self.fired:
+            raise Exception('Attempt to write more than one workbook')
+        self.fired = True
+        return self.stream
+
 class MethodFilterMethod:
 
     def __init__(self,mf,name):
