@@ -6,7 +6,7 @@
 
 from tempfile import TemporaryFile
 from unittest import TestSuite,TestCase,makeSuite
-from xlrd import open_workbook,XL_CELL_NUMBER,XL_CELL_ERROR
+from xlrd import open_workbook,XL_CELL_NUMBER,XL_CELL_ERROR,XL_CELL_TEXT
 from xlutils.filter import BaseReader,GlobReader,MethodFilter,BaseWriter,process
 from xlutils.tests.fixtures import test_files,test_xls_path,make_book,make_sheet,DummyBook
 from xlutils.tests.fixtures import compare,O,TestCallable,TestCallableMethod,TestTraversable
@@ -512,22 +512,20 @@ class TestBaseWriter(TestCase):
             self.noted_indexes[name]={}
         mapping = self.noted_indexes[name]
         a,e = getattr(ao,name),getattr(eo,name)
-        ce = mapping.get(a)
-        if ce is not None and ce!=e:
-            self.fail(
-                ('Inconsistent %s mapping, '
-                 'first: %s->%s, second: %s->%s') % (
-                name,a,ce,a,e
-                ))
-        mapping[a]=e
+        if a not in mapping:
+            mapping[a]=set()
+        # for style compression, we may get multiple expected indexes
+        # for each actual index in the output file. We just need to make
+        # sure all the data is the same.
+        mapping.get(a).add(e)
         
     def check_file(self,writer,path,
-                   l_a_xf_list=19,
-                   l_e_xf_list=22,
+                   l_a_xf_list=20,
+                   l_e_xf_list=26,
                    l_a_format_map=38,
                    l_e_format_map=37,
                    l_a_font_list=9,
-                   l_e_font_list=4):
+                   l_e_font_list=7):
         self.noted_indexes = {}
         # now open the source file
         e = open_workbook(path,pickleable=0,formatting_info=1)
@@ -628,83 +626,86 @@ class TestBaseWriter(TestCase):
         # but we check those copied are identical
         self.assertEqual(len(a.xf_list),l_a_xf_list)
         self.assertEqual(len(e.xf_list),l_e_xf_list)
-        for ai,ei in self.noted_indexes['xf_index'].items():
+        for ai,eis in self.noted_indexes['xf_index'].items():
             axf = a.xf_list[ai]
-            exf = e.xf_list[ei]
-            self.note_index(axf,exf,'format_key')
-            self.note_index(axf,exf,'font_index')
-            ap = axf.protection
-            ep = exf.protection
-            assertEqual(ap,ep,
-                        'cell_locked',
-                        'formula_hidden',
-                        )
-            ab = axf.border
-            eb = exf.border
-            assertEqual(ab,eb,
-                        'left_line_style',
-                        'right_line_style',
-                        'top_line_style',
-                        'bottom_line_style',
-                        'diag_line_style',
-                        'left_colour_index',
-                        'right_colour_index',
-                        'top_colour_index',
-                        'bottom_colour_index',
-                        'diag_colour_index',
-                        'diag_down',
-                        'diag_up',
-                        )
-            ab = axf.background
-            eb = exf.background
-            assertEqual(ab,eb,
-                        'fill_pattern',
-                        'pattern_colour_index',
-                        'background_colour_index',
-                        )
-            aa = axf.alignment
-            ea = exf.alignment
-            assertEqual(aa,ea,
-                        'hor_align',
-                        'vert_align',
-                        'text_direction',
-                        'rotation',
-                        'text_wrapped',
-                        'shrink_to_fit',
-                        'indent_level',
-                        )
+            for ei in eis:
+                exf = e.xf_list[ei]
+                self.note_index(axf,exf,'format_key')
+                self.note_index(axf,exf,'font_index')
+                ap = axf.protection
+                ep = exf.protection
+                assertEqual(ap,ep,
+                            'cell_locked',
+                            'formula_hidden',
+                            )
+                ab = axf.border
+                eb = exf.border
+                assertEqual(ab,eb,
+                            'left_line_style',
+                            'right_line_style',
+                            'top_line_style',
+                            'bottom_line_style',
+                            'diag_line_style',
+                            'left_colour_index',
+                            'right_colour_index',
+                            'top_colour_index',
+                            'bottom_colour_index',
+                            'diag_colour_index',
+                            'diag_down',
+                            'diag_up',
+                            )
+                ab = axf.background
+                eb = exf.background
+                assertEqual(ab,eb,
+                            'fill_pattern',
+                            'pattern_colour_index',
+                            'background_colour_index',
+                            )
+                aa = axf.alignment
+                ea = exf.alignment
+                assertEqual(aa,ea,
+                            'hor_align',
+                            'vert_align',
+                            'text_direction',
+                            'rotation',
+                            'text_wrapped',
+                            'shrink_to_fit',
+                            'indent_level',
+                            )
             
         # xlwt writes more formats than exist in an original,
         # but we check those copied are identical
         self.assertEqual(len(a.format_map),l_a_format_map)
         self.assertEqual(len(e.format_map),l_e_format_map)
-        for ai,ei in self.noted_indexes['format_key'].items():
+        for ai,eis in self.noted_indexes['format_key'].items():
             af = a.format_map[ai]
-            ef = e.format_map[ei]
-            assertEqual(af,ef,
-                        'format_str',
-                        'type')
+            for ei in eis:
+                ef = e.format_map[ei]
+                assertEqual(af,ef,
+                            'format_str',
+                            'type')
         # xlwt writes more fonts than exist in an original,
         # but we check those that exist in both...
         self.assertEqual(len(a.font_list),l_a_font_list)
         self.assertEqual(len(e.font_list),l_e_font_list)
-        for ai,ei in self.noted_indexes['font_index'].items():
+        for ai,eis in self.noted_indexes['font_index'].items():
             af = a.font_list[ai]
-            ef = e.font_list[ei]
-            assertEqual(af,ef,
-                        'height',
-                        'italic',
-                        'struck_out',
-                        'outline',
-                        'colour_index',
-                        'bold',
-                        'weight',
-                        'escapement',
-                        'underline_type',
-                        'family',
-                        'character_set',
-                        'name',
-                        )
+            for ei in eis:
+                ef = e.font_list[ei]
+                assertEqual(af,ef,
+                            'height',
+                            'italic',
+                            'struck_out',
+                            'outline',
+                            'colour_index',
+                            'bold',
+                            'weight',
+                            'escapement',
+                            'underline_type',
+                            'family',
+                            'character_set',
+                            'name',
+                            )
 
     def test_single_workbook_with_all_features(self):
         # create test reader
@@ -733,7 +734,7 @@ class TestBaseWriter(TestCase):
         self.failUnless('testall.xls' in w.closed)
         self.check_file(w,os.path.join(test_files,'testall.xls'))
         self.check_file(w,os.path.join(test_files,'test.xls'),
-                        18,21,38,37,8)
+                        18,21,38,37,7,4)
     
     def test_set_rd_sheet(self):
         # also tests that 'row' doesn't have to be called,
@@ -773,6 +774,22 @@ class TestBaseWriter(TestCase):
         self.assertEqual(sheet.cell(2,0).value,'S1R1C0')
         self.assertEqual(sheet.cell(3,0).value,'S2R1C0')
         
+    def test_bogus_sheet_name(self):
+        r = TestReader(
+            ('sheet',([['S1R0C0']]),),
+            ('Sheet',([['S2R0C0']]),),
+            )
+        book = tuple(r.get_workbooks())[0][0]
+        # fire methods on writer
+        self.assertRaises(ValueError,r,TestWriter())
+    
+    def test_empty_sheet_name(self):
+        r = TestReader(
+            ('',([['S1R0C0']]),),
+            )
+        book = tuple(r.get_workbooks())[0][0]
+        # fire methods on writer
+        self.assertRaises(ValueError,r,TestWriter())
     
 class TestProcess(TestCase):
 
@@ -788,6 +805,32 @@ class TestProcess(TestCase):
         F1.compare(self,[('finished',())])
         F2.compare(self,())
     
+class TestExamples(TestCase):
+
+    def test_mutate_cell(self):
+        from xlutils.filter import GlobReader,BaseFilter,process
+        class ChangeCell(BaseFilter):
+            def cell(self,rdrowx,rdcolx,wtrowx,wtcolx):
+                if self.rdsheet.name=='Sheet1' and rdrowx==0 and rdcolx==0:
+                    cell = self.rdsheet.cell(rdrowx,rdcolx)
+                    self.rdsheet.put_cell(rdrowx,rdcolx,cell.ctype,'Changed',cell.xf_index)
+                self.next.cell(rdrowx,rdcolx,wtrowx,wtcolx)
+        w = TestWriter()
+        process(
+            GlobReader(os.path.join(test_files,'testall.xls')),
+            ChangeCell(),
+            w
+            )
+        self.assertEqual(w.files.keys(),['testall.xls'])
+        f = w.files['testall.xls'].file
+        actual = open_workbook(file_contents=f.read(),pickleable=0,formatting_info=1)
+        f.close()
+        cell = actual.sheet_by_name('Sheet1').cell(0,0)
+        self.assertEqual(cell.ctype,XL_CELL_TEXT)
+        self.assertEqual(cell.value,'Changed')
+        font = actual.font_list[actual.xf_list[cell.xf_index].font_index]
+        self.assertEqual(font.underline_type,1)
+            
 def test_suite():
     return TestSuite((
         makeSuite(TestBaseReader),
@@ -800,4 +843,5 @@ def test_suite():
         makeSuite(TestColumnTrimmer),
         makeSuite(TestBaseWriter),
         makeSuite(TestProcess),
+        makeSuite(TestExamples),
         ))
